/** $lic$
 * Copyright (C) 2015-2016 by Massachusetts Institute of Technology
 *
 * This file is part of Whirltool.
 *
 * Whirltool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * If you use this software in your research, we request that you reference
 * the Whirlpool paper ("Whirlpool: Improving Dynamic Cache Management with Static Data
 * Classification", Mukkara, Beckmann and Sanchez, ASPLOS-21, April 2016) as the
 * source in any publications that use this software, and that you send us a
 * citation of your work.
 *
 * Whirltool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Whirltool.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

#ifndef POOL_MALLOC_H_
#define POOL_MALLOC_H_

#include <malloc.h>
#include <new>      // for nothrow_t
#include <stdlib.h> // for posix_memalign
#include "dlmalloc.h"

// __THROW is defined in glibc systems.  It means, counter-intuitively,
// "This function will never throw an exception."  It's an optional
// optimization tool, but we may need to use it to match glibc prototypes.
#ifndef __THROW    // I guess we're not on a glibc system
# define __THROW   // __THROW is just an optimization, so ok to make it ""
#endif

#if !HAVE_CFREE_SYMBOL
extern "C" void cfree(void* ptr) __THROW;
#endif
#if !HAVE_POSIX_MEMALIGN_SYMBOL
extern "C" int posix_memalign(void** ptr, size_t align, size_t size) __THROW;
#endif
#if !HAVE_MEMALIGN_SYMBOL
extern "C" void* memalign(size_t __alignment, size_t __size) __THROW;
#endif
#if !HAVE_VALLOC_SYMBOL
extern "C" void* valloc(size_t __size) __THROW;
#endif
#if !HAVE_PVALLOC_SYMBOL
extern "C" void* pvalloc(size_t __size) __THROW;
#endif

/* replacement routines for libc malloc */

#ifdef __cplusplus
namespace std {
    struct nothrow_t;
}
extern "C" {
#endif

void* pool_malloc(size_t) __THROW;
void pool_free(void*) __THROW;
void* pool_calloc(size_t, size_t) __THROW;
void* pool_realloc(void*, size_t) __THROW;
void* pool_memalign(size_t, size_t) __THROW;
void pool_cfree(void* ptr) __THROW;
void* pool_valloc(size_t) __THROW;
void* pool_pvalloc(size_t) __THROW;
int pool_posix_memalign(void **, size_t , size_t) __THROW;

void pool_malloc_stats(void) __THROW { }
int pool_mallopt(int cmd, int value) __THROW { }
struct mallinfo pool_mallinfo(void) __THROW { }
size_t pool_malloc_size(void* p) __THROW { }

#ifdef __cplusplus
void* pool_new(size_t);
void pool_delete(void* ptr) __THROW;
void* pool_newarray(size_t);
void pool_deletearray(void*) __THROW;
void* pool_new_nothrow(size_t, const std::nothrow_t& nt) __THROW;
void* pool_newarray_nothrow(size_t, const std::nothrow_t& nt) __THROW;
void pool_delete_nothrow(void*, const std::nothrow_t& nt) __THROW;
void pool_deletearray_nothrow(void*, const std::nothrow_t& nt) __THROW;

} // extern C
#endif

#endif // POOL_MALLOC_H_
